__version__ = '1.0.4'

from contextlib import contextmanager
from zlib import crc32

@contextmanager
def advisory_lock(lock_id, shared=False, wait=True, using=None):

    import six
    from django.db import DEFAULT_DB_ALIAS, connections, transaction

    if using is None:
        using = DEFAULT_DB_ALIAS

    # Assemble the function name based on the options.

    function_name = 'pg_'

    if not wait:
        function_name += 'try_'

    function_name += 'advisory_lock'

    if shared:
        function_name += '_shared'

    release_function_name = 'pg_advisory_unlock'
    if shared:
        release_function_name += '_shared'

    # Format up the parameters.

    tuple_format = False

    if isinstance(lock_id, (list, tuple,)):
        if len(lock_id) != 2:
            raise ValueError("Tuples and lists as lock IDs must have exactly two entries.")

        if not isinstance(lock_id[0], six.integer_types) or not isinstance(lock_id[1], six.integer_types):
            raise ValueError("Both members of a tuple/list lock ID must be integers")

        tuple_format = True
    elif isinstance(lock_id, six.string_types):
        # Generates an id within postgres integer range (-2^31 to 2^31 - 1).
        # crc32 generates an unsigned integer in Py3, we convert it into
        # a signed integer using 2's complement (this is a noop in Py2)
        pos = crc32(lock_id.encode("utf-8"))
        lock_id = (2**31 - 1) & pos
        if pos & 2**31:
            lock_id -= 2**31
    elif not isinstance(lock_id, six.integer_types):
        raise ValueError("Cannot use %s as a lock id" % lock_id)

    if tuple_format:
        base = "SELECT %s(%d, %d)"
        params = (lock_id[0], lock_id[1],)
    else:
        base = "SELECT %s(%d)"
        params = (lock_id,)

    acquire_params = ( function_name, ) + params

    command = base % acquire_params
    cursor = connections[using].cursor()

    cursor.execute(command)

    if not wait:
        acquired = cursor.fetchone()[0]
    else:
        acquired = True

    try:
        yield acquired
    finally:
        if acquired:
            release_params = ( release_function_name, ) + params

            command = base % release_params
            cursor.execute(command)

        cursor.close()



# Django settings for django_pglocks tests.

import os

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'django_pglocks',
        'USER': 'django_pglocks',
        'PASSWORD': 'django_pglocks',
    }
}

INSTALLED_APPS = ['django_pglocks']

SECRET_KEY = 'whatever'



from django.db import connection
from django.test import TransactionTestCase

from django_pglocks import advisory_lock


class PgLocksTests(TransactionTestCase):

    @classmethod
    def setUpClass(cls):
        cursor = connection.cursor()
        cursor.execute(
                "SELECT oid FROM pg_database WHERE datname = %s",
                [connection.settings_dict['NAME']])
        cls.db_oid = cursor.fetchall()[0][0]
        cursor.close()

    def assertNumLocks(self, expected):
        cursor = connection.cursor()
        cursor.execute(
                "SELECT COUNT(*) FROM pg_locks WHERE database = %s AND locktype = %s",
                [self.db_oid, 'advisory'])
        actual = cursor.fetchall()[0][0]
        cursor.close()
        self.assertEqual(actual, expected)

    def test_basic_lock_str(self):
        self.assertNumLocks(0)
        with advisory_lock('test') as acquired:
            self.assertTrue(acquired)
            self.assertNumLocks(1)
        self.assertNumLocks(0)

    def test_basic_lock_int(self):
        self.assertNumLocks(0)
        with advisory_lock(123) as acquired:
            self.assertTrue(acquired)
            self.assertNumLocks(1)
        self.assertNumLocks(0)

    def test_basic_lock_tuple(self):
        self.assertNumLocks(0)
        with advisory_lock((123, 456)) as acquired:
            self.assertTrue(acquired)
            self.assertNumLocks(1)
        self.assertNumLocks(0)

    def test_basic_lock_no_wait(self):
        self.assertNumLocks(0)
        with advisory_lock(123, wait=False) as acquired:
            self.assertTrue(acquired)
            self.assertNumLocks(1)
        self.assertNumLocks(0)

    def test_basic_lock_shared(self):
        self.assertNumLocks(0)
        with advisory_lock(123, shared=True) as acquired:
            self.assertTrue(acquired)
            self.assertNumLocks(1)
        self.assertNumLocks(0)

    def test_basic_lock_shared_no_wait(self):
        self.assertNumLocks(0)
        with advisory_lock(123, shared=True, wait=False) as acquired:
            self.assertTrue(acquired)
            self.assertNumLocks(1)
        self.assertNumLocks(0)



#!/usr/bin/env python

from distutils.core import setup

import django_pglocks

def get_long_description():
    """
    Return the contents of the README file.
    """
    try:
        return open('README.rst').read()
    except:
        pass  # Required to install using pip (won't have README then)

setup(
    name = 'django-pglocks',
    version = django_pglocks.__version__,
    description = "django_pglocks provides useful context managers for advisory locks for PostgreSQL.",
    long_description = get_long_description(),
    author = "Christophe Pettus",
    author_email = "xof@thebuild.com",
    license = "MIT",
    url = "https://github.com/Xof/django-pglocks",
    install_requires = ['six>=1.0.0'],
    packages = [
        'django_pglocks',
    ],
    package_data = {
        'facetools': ['templates/facetools/facebook_redirect.html'],
    },
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Software Development',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ]
)
